@@ -23,10 +23,8 @@
 #ifndef GNOME_THEME_INFO_H
 #define GNOME_THEME_INFO_H
 
-#include <config.h>
-
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdk.h>
 
@@ -130,13 +128,25 @@ typedef void (* ThemeChangedCallback) (GnomeThemeCommonInfo *theme,
 				       GnomeThemeChangeType  change_type,
 				       gpointer              user_data);
 
+#define GNOME_THEME_ERROR gnome_theme_info_error_quark ()
+
+enum {
+  GNOME_THEME_ERROR_GTK_THEME_NOT_AVAILABLE = 1,
+  GNOME_THEME_ERROR_WM_THEME_NOT_AVAILABLE,
+  GNOME_THEME_ERROR_ICON_THEME_NOT_AVAILABLE,
+  GNOME_THEME_ERROR_GTK_ENGINE_NOT_AVAILABLE,
+  GNOME_THEME_ERROR_UNKNOWN
+};
+
 
 /* GTK/Metacity/keybinding Themes */
 GnomeThemeInfo     *gnome_theme_info_new                   (void);
 void                gnome_theme_info_free                  (GnomeThemeInfo     *theme_info);
 GnomeThemeInfo     *gnome_theme_info_find                  (const gchar        *theme_name);
 GList              *gnome_theme_info_find_by_type          (guint               elements);
-
+GQuark              gnome_theme_info_error_quark           (void);
+gchar              *gtk_theme_info_missing_engine          (const gchar *gtk_theme,
+                                                            gboolean nameOnly);
 
 /* Icon Themes */
 GnomeThemeIconInfo *gnome_theme_icon_info_new              (void);
@@ -157,15 +167,16 @@ gint                  gnome_theme_cursor_info_compare      (GnomeThemeCursorInfo
 /* Meta themes*/
 GnomeThemeMetaInfo *gnome_theme_meta_info_new              (void);
 void                gnome_theme_meta_info_free             (GnomeThemeMetaInfo *meta_theme_info);
-void                gnome_theme_meta_info_print            (GnomeThemeMetaInfo *meta_theme_info);
 GnomeThemeMetaInfo *gnome_theme_meta_info_find             (const gchar        *meta_theme_name);
 GList              *gnome_theme_meta_info_find_all         (void);
 gint                gnome_theme_meta_info_compare          (GnomeThemeMetaInfo *a,
 							    GnomeThemeMetaInfo *b);
-GnomeThemeMetaInfo *gnome_theme_read_meta_theme            (GnomeVFSURI        *meta_theme_uri);
+gboolean            gnome_theme_meta_info_validate         (const GnomeThemeMetaInfo *info,
+                                                            GError            **error);
+GnomeThemeMetaInfo *gnome_theme_read_meta_theme            (GFile              *meta_theme_uri);
 
 /* Other */
-void                gnome_theme_init                       (gboolean            *monitor_not_added);
+void                gnome_theme_init                       (void);
 void                gnome_theme_info_register_theme_change (ThemeChangedCallback func,
 							    gpointer             data);
 
